using System;

using System.Collections.Generic;

using System.Text;

using Autodesk.AutoCAD.ApplicationServices;

using Autodesk.AutoCAD.DatabaseServices;

using Autodesk.AutoCAD.GraphicsInterface;

using Autodesk.AutoCAD.Runtime;

using LispExtensionTools;


namespace LispPurge

{

    public class LispFun

    {

        /* 1        Blocs

         * 2        Calques

         * 4        Formes

         * 8        Matriaux

         * 16       Styles de cote

         * 32       Styles de ligne de repre multiple

         * 64       Styles de mligne

         * 128      Styles de tableau

         * 256      Styles de texte

         * 512      Styles de trac

         * 1024     Styles visuels

         * 2048     Type de ligne

         * 4096     Applications enregistres

         * 8192     Goupes

         * 16384        Echelles

         * 32768        Textes vides

         * 65536        Entits de longueur nulle

         * 131072   Images raster

         * 262144       DWF sous-jascents

         * 524288       PDF sous-jascents

         * 1048576      DGN sous-jascents

        */


        [LispFunction("gc-purge")]

        public TypedValue Purge(ResultBuffer resbuf)

        {

            TypedValue nil = new TypedValue((int)LispDataType.Nil);

            if (resbuf == null)

            {

                LispErrorMsg.TooFew();

                return nil;

            }

            TypedValue[] tvs = resbuf.AsArray();

            if (tvs.Length == 0)

            {

                LispErrorMsg.TooFew();

                return nil;

            }

            if (tvs.Length > 1)

            {

                LispErrorMsg.TooMuch();

                return nil;

            }

            TypedValue tv = tvs[0];

            if (tv.TypeCode != (int)LispDataType.Int16 && tv.TypeCode != (int)LispDataType.Int32)

            {

                LispErrorMsg.BadType("fixnump", tv);

                return nil;

            }

            int flag = Convert.ToInt32(tv.Value);

            try

            {

                PurgeAll(HostApplicationServices.WorkingDatabase, flag);

                return new TypedValue((int)LispDataType.T_atom);

            }

            catch (Autodesk.AutoCAD.Runtime.Exception ex)

            {

                Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage(

                    "\nErreur application: " + ex.Message + ex.StackTrace);

                return nil;

            }

            catch (System.Exception ex)

            {

                Application.DocumentManager.MdiActiveDocument.Editor.WriteMessage(

                    "\nErreur systme: " + ex.Message + ex.StackTrace);

                return nil;

            }

        }


        private void PurgeAll(Database db, int flag)

        {

            ObjectIdCollection tableIds = new ObjectIdCollection();

            ObjectIdCollection dictIds = new ObjectIdCollection();

            if ((flag & 1) == 1)

                tableIds.Add(db.BlockTableId);

            if ((flag & 2) == 2)

                tableIds.Add(db.LayerTableId);

            if ((flag & 16) == 16)

                tableIds.Add(db.DimStyleTableId);

            if ((flag & 8) == 8)

                tableIds.Add(db.LinetypeTableId);

            if ((flag & 256) == 256)

                tableIds.Add(db.TextStyleTableId);

            if ((flag & 4096) == 4096)

                tableIds.Add(db.RegAppTableId);

            if ((flag & 64) == 64)

                dictIds.Add(db.MLStyleDictionaryId);

            if ((flag & 512) == 512)

                dictIds.Add(db.PlotStyleNameDictionaryId);

            if ((flag & 128) == 128)

                dictIds.Add(db.TableStyleDictionaryId);

            bool emptyTxt = (flag & 32768) == 32768;

            bool zeroLength = (flag & 65536) == 65536;

            if (emptyTxt || zeroLength)

            {

                using (Transaction tr = db.TransactionManager.StartTransaction())

                {

                    List locked = new List();

                    LayerTable lt = (LayerTable)tr.GetObject(db.LayerTableId, OpenMode.ForRead);

                    foreach (ObjectId layerId in lt)

                    {

                        LayerTableRecord ltr = (LayerTableRecord)tr.GetObject(layerId, OpenMode.ForRead);

                        if (ltr.IsLocked == true)

                        {

                            ltr.UpgradeOpen();

                            ltr.IsLocked = false;

                            locked.Add(ltr);

                        }

                    }

                    BlockTable bt = (BlockTable)tr.GetObject(db.BlockTableId, OpenMode.ForRead);

                    foreach (ObjectId btrId in bt)

                    {

                        BlockTableRecord btr =

                            (BlockTableRecord)tr.GetObject(btrId, OpenMode.ForRead);

                        if (!btr.IsLayout)

                            continue;

                        foreach (ObjectId id in btr)

                        {

                            if (zeroLength)

                            {

                                Curve crv = tr.GetObject(id, OpenMode.ForRead) as Curve;

                                if (crv != null)

                                {

                                    if (crv.GetDistanceAtParameter(crv.EndParam) == 0.0)

                                    {

                                        crv.UpgradeOpen();

                                        crv.Erase();

                                    }

                                    continue;

                                }

                            }

                            if (emptyTxt)

                            {

                                DBText txt = tr.GetObject(id, OpenMode.ForRead) as DBText;

                                if (txt != null)

                                {

                                    if (txt.TextString.Trim() == string.Empty)

                                    {

                                        txt.UpgradeOpen();

                                        txt.Erase();

                                    }

                                    continue;

                                }

                                MText mTxt = tr.GetObject(id, OpenMode.ForRead) as MText;

                                if (mTxt != null && GetStripedMtextContents(mTxt).Trim() == string.Empty)

                                {

                                    mTxt.UpgradeOpen();

                                    mTxt.Erase();

                                }

                            }

                        }

                    }

                    foreach (LayerTableRecord ltr in locked)

                    {

                        ltr.IsLocked = true;

                    }

                    tr.Commit();

                }

            }

            while (PurgeDatabase(db, tableIds, dictIds, flag))

                continue;

        }



        private string m_str;


        private string GetStripedMtextContents(MText mt)

        {

            m_str = string.Empty;

            mt.ExplodeFragments(new MTextFragmentCallback(FragmentCallback));

            return m_str;

        }


        private MTextFragmentCallbackStatus FragmentCallback(MTextFragment fragment, object obj)

        {

            m_str += fragment.Text;

            return MTextFragmentCallbackStatus.Continue;

        }


        private bool PurgeDatabase(Database db, ObjectIdCollection tableIds, ObjectIdCollection dictIds, int flag)

        {

            ObjectIdCollection ids = new ObjectIdCollection();

            using (Transaction tr = db.TransactionManager.StartTransaction())

            {

                if ((flag & 8192) == 8192)

                {

                    DBDictionary dict =

                        (DBDictionary)tr.GetObject(db.GroupDictionaryId, OpenMode.ForRead, false);

                    foreach (DBDictionaryEntry entry in dict)

                    {

                        Group grp = (Group)tr.GetObject(entry.Value, OpenMode.ForRead, false);

                        if (grp.NumEntities == 0)

                        {

                            grp.UpgradeOpen();

                            grp.Erase();

                        }

                    }

                }

                DBDictionary NOD =

                        (DBDictionary)tr.GetObject(db.NamedObjectsDictionaryId, OpenMode.ForRead);

                Dictionary dicts = new Dictionary();

                dicts.Add(32, "ACAD_MLEADERSTYLE");

                dicts.Add(262144, "ACAD_DWFDEFINITIONS");

                dicts.Add(524288, "ACAD_PDFDEFINITIONS");

                dicts.Add(1048576, "ACAD_DGNDEFINITIONS");

                foreach (KeyValuePair pair in dicts)

                {

                    if ((flag & pair.Key) == pair.Key && NOD.Contains(pair.Value))

                        dictIds.Add(NOD.GetAt(pair.Value));

                }

                foreach (ObjectId tableId in tableIds)

                {

                    SymbolTable table = (SymbolTable)tr.GetObject(tableId, OpenMode.ForRead, false);

                    foreach (ObjectId recordId in table)

                    {

                        ids.Add(recordId);

                    }

                }

                foreach (ObjectId dictId in dictIds)

                {

                    DBDictionary dict = (DBDictionary)tr.GetObject(dictId, OpenMode.ForRead, false);

                    foreach (DBDictionaryEntry entry in dict)

                    {

                        if (entry.Value.IsValid)

                        {

                            DBObject obj = (DBObject)tr.GetObject(entry.Value, OpenMode.ForRead);

                            if (!obj.IsAProxy)

                                ids.Add(entry.Value);

                        }

                    }

                }

                if ((flag & 131072) == 131072 && NOD.Contains("ACAD_IMAGE_DICT"))

                {

                    DBDictionary dict = (DBDictionary)tr.GetObject(NOD.GetAt("ACAD_IMAGE_DICT"), OpenMode.ForRead);

                    foreach (DBDictionaryEntry entry in dict)

                    {

                        if (entry.Value.IsValid)

                        {

                            bool lck;

                            RasterImageDef obj = (RasterImageDef)tr.GetObject(entry.Value, OpenMode.ForRead);

                            if (!obj.IsAProxy && obj.GetEntityCount(out lck) == 0)

                                ids.Add(entry.Value);

                        }

                    }

                }

                if ((flag & 16384) == 16384 && NOD.Contains("ACAD_SCALELIST"))

                {

                    DBDictionary dict = (DBDictionary)tr.GetObject(NOD.GetAt("ACAD_SCALELIST"), OpenMode.ForRead);

                    foreach (DBDictionaryEntry entry in dict)

                    {

                        if (entry.Key != "A0")

                        {

                            DBObject obj = (DBObject)tr.GetObject(entry.Value, OpenMode.ForRead);

                            if (!obj.IsAProxy)

                                ids.Add(entry.Value);

                        }

                    }

                }

                if ((flag & 8) == 8)

                {

                    DBDictionary matDict =

                        (DBDictionary)tr.GetObject(db.MaterialDictionaryId, OpenMode.ForRead, false);

                    foreach (DBDictionaryEntry entry in matDict)

                    {

                        string key = entry.Key;

                        if ((key != "ByBlock") && (key != "ByLayer") && (key != "Global"))

                        {

                            DBObject obj = (DBObject)tr.GetObject(entry.Value, OpenMode.ForRead);

                            if (!obj.IsAProxy)

                                ids.Add(entry.Value);

                        }

                    }

                }

                if ((flag & 1024) == 1024)

                {

                    DBDictionary vsDict =

                        (DBDictionary)tr.GetObject(db.VisualStyleDictionaryId, OpenMode.ForRead);

                    foreach (DBDictionaryEntry entry in vsDict)

                    {

                        DBVisualStyle vs = tr.GetObject(entry.Value, OpenMode.ForRead) as DBVisualStyle;

                        if (vs.Type == VisualStyleType.Custom)

                        {

                            DBObject obj = (DBObject)tr.GetObject(entry.Value, OpenMode.ForRead);

                            if (!obj.IsAProxy)

                                ids.Add(entry.Value);

                        }

                    }

                }

                if ((flag & 4) == 4)

                {

                    TextStyleTable tst =

                        (TextStyleTable)tr.GetObject(db.TextStyleTableId, OpenMode.ForRead);

                    foreach (ObjectId id in tst)

                    {

                        TextStyleTableRecord tstr =

                            (TextStyleTableRecord)tr.GetObject(id, OpenMode.ForRead);

                        if (tstr.IsShapeFile && !tstr.IsAProxy)

                            ids.Add(id);

                    }

                }

                db.Purge(ids);

                foreach (ObjectId id in ids)

                {

                    DBObject obj = (DBObject)tr.GetObject(id, OpenMode.ForWrite);

                    obj.Erase();

                }

                tr.Commit();

            }

            return ids.Count > 0;

        }

    }

}

using Autodesk.AutoCAD.ApplicationServices;

using Autodesk.AutoCAD.DatabaseServices;

using Autodesk.AutoCAD.EditorInput;

using Autodesk.AutoCAD.Runtime;


namespace LispExtensionTools

{

    public static class LispErrorMsg

    {

        static Editor ed = Application.DocumentManager.MdiActiveDocument.Editor;


        internal static void TooFew()

        {

            ed.WriteMessage("\nErreur: nombre d'arguments insuffisants\n");

        }


        internal static void TooMuch()

        {

            ed.WriteMessage("\nErreur: nombre d'arguments trop important\n");

        }


        internal static void BadType(string T, TypedValue tV)

        {

            string msg = tV.TypeCode == (int)LispDataType.Nil ? "nil" : tV.Value.ToString();

            ed.WriteMessage("\nErreur: type d'argument incorrect: {0} {1}\n",T , msg);

        }


        internal static void Message(string msg)

        {

            ed.WriteMessage("\nErreur: {0}\n", msg);

        }

    }

}
